package org.processmining.logenhancement.abstraction;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.table.DefaultTableModel;

import org.deckfour.xes.model.XEvent;
import org.processmining.framework.util.ui.widgets.ColorScheme;
import org.processmining.framework.util.ui.widgets.ProMTableWithoutPanel;
import org.processmining.framework.util.ui.widgets.traceview.ProMTraceList.ClickListener;
import org.processmining.framework.util.ui.widgets.traceview.ProMTraceList.TraceBuilder;
import org.processmining.framework.util.ui.widgets.traceview.ProMTraceView.Event;
import org.processmining.framework.util.ui.widgets.traceview.ProMTraceView.Trace;
import org.processmining.logenhancement.abstraction.model.AbstractionPattern;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.plugins.DataConformance.visualization.alignment.AlignmentListView;
import org.processmining.plugins.DataConformance.visualization.alignment.AlignmentListView.XAlignmentOrdering;
import org.processmining.plugins.DataConformance.visualization.alignment.AttributesComparisonPanel;
import org.processmining.plugins.DataConformance.visualization.alignment.XAlignmentEvent;
import org.processmining.xesalignmentextension.XAlignmentExtension;
import org.processmining.xesalignmentextension.XAlignmentExtension.MoveType;
import org.processmining.xesalignmentextension.XAlignmentExtension.XAlignedLog;
import org.processmining.xesalignmentextension.XAlignmentExtension.XAlignment;
import org.processmining.xesalignmentextension.XAlignmentExtension.XAlignmentMove;
import org.processmining.xesalignmentextension.XDataAlignmentExtension;
import org.processmining.xesalignmentextension.XDataAlignmentExtension.DataMoveType;
import org.processmining.xesalignmentextension.XDataAlignmentExtension.XDataAlignmentExtensionException;
import org.processmining.xesalignmentextension.XDataAlignmentExtension.XDataAlignmentMove;

import com.fluxicon.slickerbox.factory.SlickerFactory;
import com.google.common.base.Function;
import com.google.common.base.Predicate;
import com.google.common.collect.Iterators;
import com.google.common.collect.UnmodifiableIterator;

public class AlignmentGuidedPatternResult {

	private static final DecimalFormat FLOAT_FORMAT = new DecimalFormat("##0.00%");

	private final class AlignmentTraceWithAbstractionInfo implements Trace<Event> {

		private final XAlignment alignment;

		private final Map<String, Color> activityColorMap;
		private final Map<String, AbstractionPattern> transitionIdToPattern;

		public AlignmentTraceWithAbstractionInfo(final XAlignment alignment,
				Map<String, AbstractionPattern> transitionIdToPattern, Map<String, Color> activityColorMap) {
			this.alignment = alignment;
			this.transitionIdToPattern = transitionIdToPattern;
			this.activityColorMap = activityColorMap;
		}

		public String getName() {
			String name = alignment.getName();
			return "Trace: ".concat(name != null ? name : "unkown trace");
		}

		public Color getNameColor() {
			return null;
		}

		public String getInfo() {
			return "Fitness ".concat(FLOAT_FORMAT.format(alignment.getFitness()));
		}

		public Color getInfoColor() {
			return null;
		}

		public Iterator<Event> iterator() {
			UnmodifiableIterator<XAlignmentMove> observableMoves = Iterators.filter(alignment.iterator(),
					new Predicate<XAlignmentMove>() {

						public boolean apply(XAlignmentMove move) {
							return move.isObservable();
						}
					});
			return Iterators.transform(observableMoves, new Function<XAlignmentMove, Event>() {

				public Event apply(XAlignmentMove move) {
					final XDataAlignmentMove dataMove = XDataAlignmentExtension.instance().extendXAlignmentMove(move);
					return new XAlignmentEvent(dataMove, activityColorMap, true) {

						public String getLabel() {
							AbstractionPattern abstractionPattern = transitionIdToPattern.get(dataMove.getActivityId());
							if (abstractionPattern != null) {
								return "[" + abstractionPattern.getPatternName() + "] " + super.getLabel();
							} else {
								return super.getLabel();
							}
						}

					};
				}

			});
		}

	}

	private final XAlignedLog alignedLog;

	private final Map<String, AbstractionPattern> transitionIdToPattern = new HashMap<>();
	private final Map<AbstractionPattern, Double> errorRate = new HashMap<>();
	private final Map<AbstractionPattern, Long> numMatches = new HashMap<>();

	public AlignmentGuidedPatternResult(XAlignedLog alignedLog, Collection<AbstractionPattern> patternMap) {
		this.alignedLog = alignedLog;

		for (AbstractionPattern pattern : patternMap) {
			long matchCount = 0;
			long moveCount = 0;
			long modelMoveCount = 0;
			for (String id : pattern.getTransitionsAsLocalIds()) {
				transitionIdToPattern.put(id, pattern);
			}
			Transition startTransition = pattern.getStartTransition();
			Set<String> transitions = pattern.getTransitionsAsLocalIds();
			String startTransitionId = startTransition.getLocalID().toString();
			for (XAlignment alignment : alignedLog) {
				for (XAlignmentMove move : alignment) {
					String activityId = move.getActivityId();
					if (startTransitionId.equals(activityId)) {
						matchCount++;
					}
					if (move.isObservable() && transitions.contains(move.getActivityId())) {
						moveCount++;
						if (move.getType() != MoveType.SYNCHRONOUS) {
							modelMoveCount++;
						}
					}
				}
			}
			numMatches.put(pattern, matchCount);
			if (moveCount != 0) {
				double error = modelMoveCount / (double) moveCount;
				errorRate.put(pattern, error);
			} else {
				errorRate.put(pattern, 0.0d);
			}
		}
	}

	public JComponent getPanel() {
		final AlignmentListView alignmentView = createAlignmentView();
		final JFrame popupDialog = new JFrame();
		popupDialog.setPreferredSize(new Dimension(800, 600));
		popupDialog.add(alignmentView);
		popupDialog.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
		popupDialog.pack();

		final JPanel panel = new JPanel() {

			private static final long serialVersionUID = 1L;

			public void removeNotify() {
				popupDialog.setVisible(false);
				popupDialog.dispose();
				super.removeNotify();
			}

		};
		panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));

		JButton showAlignment = SlickerFactory.instance()
				.createButton("Show aligned matches between events and activity patterns");
		showAlignment.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				popupDialog.setLocationRelativeTo(panel);
				popupDialog.setAlwaysOnTop(false);
				popupDialog.setVisible(true);
			}
		});
		panel.add(showAlignment);

		DefaultTableModel statsModel = new DefaultTableModel();
		ProMTableWithoutPanel statsTable = new ProMTableWithoutPanel(statsModel);
		panel.add(statsTable);

		statsModel.addColumn("Name");
		statsModel.addColumn("Value");

		NumberFormat integerInstance = NumberFormat.getIntegerInstance();
		NumberFormat percentInstance = NumberFormat.getPercentInstance();

		statsModel.addRow(new String[] { "# Total Matches", integerInstance.format(getNumMatches()) });
		statsModel.addRow(new String[] { "% Fitness", percentInstance.format(alignedLog.getAverageFitness()) });
		statsModel.addRow(new String[] { "% Total Error (Model Move Ratio)", percentInstance.format(getErrorRate()) });
		statsModel.addRow(new String[] { "Individual  activity patterns", "-" });

		for (Entry<AbstractionPattern, Long> entry : numMatches.entrySet()) {
			statsModel.addRow(new String[] { "# Matches '" + entry.getKey().getPatternName() + "'",
					integerInstance.format(entry.getValue()) });
			statsModel.addRow(new String[] { "% Error '" + entry.getKey().getPatternName() + "'",
					percentInstance.format(errorRate.get(entry.getKey())) });
		}

		return panel;
	}

	private AlignmentListView createAlignmentView() {

		final Map<String, Color> colorMap = createColorMap(numMatches);
		final AlignmentListView alignmentView = new AlignmentListView(alignedLog, XAlignmentOrdering.FITNESS_ASC,
				new TraceBuilder<XAlignment>() {

					public Trace<Event> build(XAlignment a) {
						return new AlignmentTraceWithAbstractionInfo(a, transitionIdToPattern, colorMap);
					}
				});
		alignmentView.setOpaque(true);
		alignmentView.setBackground(Color.WHITE);
		alignmentView.setMaxWedgeWidth(100);
		alignmentView.setPreferredSize(new Dimension(600, 400));
		alignmentView.sort(XAlignmentOrdering.FITNESS_DESC);
		alignmentView.addTraceClickListener(new ClickListener<XAlignment>() {

			public void traceMouseDoubleClicked(XAlignment a, int traceIndex, int eventIndex, MouseEvent e) {

				if (eventIndex != -1) {

					XEvent alignmentEvent = a.getTrace().get(eventIndex);
					XAlignmentMove move = XAlignmentExtension.instance().extendEvent(alignmentEvent);
					DataMoveType dataMoveType = XDataAlignmentExtension.instance().extractDataMoveType(alignmentEvent);

					String eventName = move.getModelMove() != null ? move.getModelMove() : move.getLogMove();

					AttributesComparisonPanel comparisonPanel = new AttributesComparisonPanel();
					try {
						comparisonPanel.displayAlignmentMove(move);

						JDialog popupDialog = new JDialog();
						popupDialog.setPreferredSize(new Dimension(600, 300));
						if (dataMoveType == null) {
							popupDialog.setTitle(eventName + "(" + move.getType() + ")");
						} else {
							popupDialog.setTitle(eventName + "(" + move.getType() + "/" + dataMoveType + ")");
						}
						popupDialog.add(comparisonPanel);
						popupDialog.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
						popupDialog.pack();
						popupDialog.setLocationRelativeTo(alignmentView);
						popupDialog.setAlwaysOnTop(true);
						popupDialog.setVisible(true);

					} catch (XDataAlignmentExtensionException e1) {
						Object[] options = { "OK" };
						JOptionPane.showOptionDialog(alignmentView, e1.getMessage(), "Error", JOptionPane.PLAIN_MESSAGE,
								JOptionPane.ERROR_MESSAGE, null, options, options[0]);
					}

				}
			}

			public void traceMouseClicked(XAlignment a, int traceIndex, int eventIndex, MouseEvent e) {
			}

		});
		return alignmentView;
	}

	private Map<String, Color> createColorMap(Map<AbstractionPattern, Long> numMatches) {
		Map<String, Color> activityColorMap = new HashMap<>();
		int i = 0;
		for (AbstractionPattern pattern : numMatches.keySet()) {
			Color color = ColorScheme.COLOR_BREWER_12CLASS_PAIRED.getColor(i++);
			for (String localId : pattern.getTransitionsAsLocalIds()) {
				activityColorMap.put(localId, color);
			}
		}
		return activityColorMap;
	}

	public Double getErrorRate() {
		double total = 0.0d;
		int count = errorRate.size();
		for (Double val : errorRate.values()) {
			total += val;
		}
		return total / count;
	}

	public Long getNumMatches() {
		long total = 0;
		for (Long val : numMatches.values()) {
			total += val;
		}
		return total;
	}

}
