package org.processmining.logenhancement.cpn;

import java.math.BigInteger;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;
import java.util.TreeMap;
import java.util.TreeSet;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.deckfour.xes.extension.std.XConceptExtension;
import org.deckfour.xes.extension.std.XLifecycleExtension;
import org.deckfour.xes.extension.std.XOrganizationalExtension;
import org.deckfour.xes.extension.std.XTimeExtension;
import org.deckfour.xes.factory.XFactory;
import org.deckfour.xes.model.XAttribute;
import org.deckfour.xes.model.XAttributeBoolean;
import org.deckfour.xes.model.XAttributeContinuous;
import org.deckfour.xes.model.XAttributeDiscrete;
import org.deckfour.xes.model.XAttributeLiteral;
import org.deckfour.xes.model.XAttributeTimestamp;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.plugins.utils.ProvidedObjectHelper;

/**
 * @author michael
 * 
 */
class KeyValueToLog {

	enum AttributeKind {
		LITERAL("Literal (string)", XAttributeLiteral.class), DATE("Date/time", XAttributeTimestamp.class), BOOLEAN(
				"Boolean", XAttributeBoolean.class), DISCRETE("Discrete (integer)", XAttributeDiscrete.class), CONTINUOUS(
				"Continuous (double)", XAttributeContinuous.class);

		private final Class<? extends XAttribute> clazz;
		private final String name;

		private AttributeKind(final String name, final Class<? extends XAttribute> clazz) {
			this.name = name;
			this.clazz = clazz;
		}

		public Class<? extends XAttribute> getClazz() {
			return clazz;
		}

		@Override
		public String toString() {
			return name;
		}
	}

	private static final Pattern pattern = Pattern.compile("\\s*(\\d+)-(\\d+)-(\\d+)(\\s*(\\d+):(\\d+)(:(\\d+))?)?");

	public static Object[] keyValueToLog(final PluginContext context, XFactory factory, Mapping mapping, final KeyValue... set) {
		if (set == null || set.length < 1) {
			context.getFutureResult(0).cancel(true);
			return null;
		}
		XLog resultLog = null;
		context.log("Converting " + set.length + " log" + (set.length == 1 ? "" : "s") + "...");
		int n = 0;
		for (KeyValue s : set) {
			n++;
			context.log((set.length > 1 ? ("Log " + n + " / " + set.length + ": ") : "") + "Found " + s.size()
					+ " data item" + (s.size() != 1 ? "s" : ""));
			context.getProgress().setIndeterminate(false);
			context.getProgress().setMaximum(s.size() * 3);

			int i = 0;

			final Map<Object, Set<XEvent>> traces = new TreeMap<Object, Set<XEvent>>(new Comparator<Object>() {
				@Override
				@SuppressWarnings("unchecked")
				public int compare(final Object arg0, final Object arg1) {
					if (arg0 == null)
						return arg1 == null ? 0 : 1;
					if (arg1 == null)
						return -1;
					if (arg0 instanceof Comparable) {
						try {
							return ((Comparable<Object>) arg0).compareTo(arg1);
						} catch (final Exception _) {
						}
					}
					if (arg1 instanceof Comparable) {
						try {
							return -((Comparable<Object>) arg1).compareTo(arg0);
						} catch (final Exception _) {
						}
					}
					if (arg0.equals(arg1))
						return 0;
					final int result = arg0.getClass().toString().compareTo(arg1.getClass().toString());
					if (result == 0)
						return arg0.hashCode() - arg1.hashCode();
					return result;
				}
			});
			final Map<XEvent, Object> sorting = new HashMap<XEvent, Object>();
			for (final Map<String, Object> item : s) {
				Object key = KeyValueToLog.get(mapping.getTraceID(), item);
				if (key == null) {
					key = Boolean.TRUE;
				}
				if (!traces.containsKey(key)) {
					final TreeSet<XEvent> trace = new TreeSet<XEvent>(new Comparator<XEvent>() {
						@Override
						public int compare(final XEvent arg0, final XEvent arg1) {
							if (arg0 == null)
								return arg1 == null ? 0 : -1;
							if (arg1 == null)
								return 1;
							final Date date0 = XTimeExtension.instance().extractTimestamp(arg0);
							final Date date1 = XTimeExtension.instance().extractTimestamp(arg1);
							if (date0 == null)
								return date1 == null ? 1 : -1;
							if (date1 == null)
								return 1;
							int result = date0.compareTo(date1);
							if (result != 0)
								return result;
							result = compareSorting(sorting.get(arg0), sorting.get(arg1));
							if (result != 0)
								return result;
							result = localComp(XConceptExtension.instance().extractName(arg0), XConceptExtension
									.instance().extractName(arg1));
							if (result != 0)
								return result;
							result = localComp(XLifecycleExtension.instance().extractStandardTransition(arg0),
									XLifecycleExtension.instance().extractStandardTransition(arg1));
							if (result != 0)
								return result;
							result = localComp(XLifecycleExtension.instance().extractTransition(arg0),
									XLifecycleExtension.instance().extractTransition(arg1));
							if (result != 0)
								return result;
							result = localComp(XOrganizationalExtension.instance().extractResource(arg0),
									XOrganizationalExtension.instance().extractResource(arg1));
							if (result != 0)
								return result;
							return 0;
							//						return arg1.hashCode() - arg1.hashCode();
						}

						@SuppressWarnings("unchecked")
						private int compareSorting(Object object, Object object2) {
							if (object == null)
								return object2 == null ? 0 : -1;
							if (object2 == null)
								return 1;
							try {
								return ((Comparable<Object>) object).compareTo(object2);
							} catch (Exception _) {
								// Ignore, this happens if it is not comparable or not comparable to the type of object2
							}
							try {
								return -1 * (((Comparable<Object>) object2).compareTo(object));
							} catch (Exception _) {
								// Ignore, same reason as above
							}
							return 0;
						}

						public <T extends Comparable<T>> int localComp(final T a, final T b) {
							if (a == null)
								return b == null ? 0 : -1;
							if (b == null)
								return 1;
							return a.compareTo(b);
						}
					});
					traces.put(key, trace);
				}
				final Set<XEvent> trace = traces.get(key);
				final XEvent event = factory.createEvent();
				for (final Entry<XAttribute, List<String>> entry : mapping.getMapping().entrySet()) {
					final Object value = KeyValueToLog.get(entry.getValue(), item);
					if (value != null) {
						final XAttribute convertValue = KeyValueToLog.convertValue(entry.getKey(), value);
						if (convertValue != null) {
							event.getAttributes().put(convertValue.getKey(), convertValue);
						}
					}
				}
				sorting.put(event, get(mapping.getSorting(), item));
				trace.add(event);

				context.getProgress().setValue(i += 2);
			}
			sorting.clear();
			context.log("Found " + traces.size() + " trace" + (traces.size() != 1 ? "s" : ""));

			final XLog result = factory.createLog();
			for (final Entry<Object, Set<XEvent>> trace : traces.entrySet()) {
				final XTrace t = factory.createTrace();
				t.getAttributes().put(XConceptExtension.ATTR_NAME.getKey(),
						KeyValueToLog.convertValue(XConceptExtension.ATTR_NAME, trace.getKey()));
				t.addAll(trace.getValue());
				result.add(t);
				context.getProgress().setValue(i += t.size());
			}
			if (n == 1) {
				resultLog = result;
			} else {
				ProvidedObjectHelper.publish(context, "Log " + n, result, XLog.class, true);
			}
		}
		return new Object[] { resultLog };
	}


	private static XAttribute convertValue(final XAttribute key, final Object value) {
		if (value == null)
			return null;
		final XAttribute clone = (XAttribute) key.clone();
		if (key instanceof XAttributeBoolean) {
			if (value instanceof Boolean) {
				((XAttributeBoolean) clone).setValue((Boolean) value);
				return clone;
			}
		}
		if (key instanceof XAttributeTimestamp) {
			if (value instanceof Date) {
				((XAttributeTimestamp) clone).setValue((Date) value);
				return clone;
			}
			if (value instanceof Long) {
				((XAttributeTimestamp) clone).setValueMillis((Long) value);
				return clone;
			}
			if (value instanceof Integer) {
				((XAttributeTimestamp) clone).setValueMillis((Integer) value);
				return clone;
			}
			if (value instanceof BigInteger) {
				((XAttributeTimestamp) clone).setValueMillis(((BigInteger) value).longValue());
				return clone;
			}
			if (value instanceof String) {
				final Date date = KeyValueToLog.datify((String) value);
				if (date != null) {
					((XAttributeTimestamp) clone).setValue(date);
					return clone;
				}
			}
		}
		if (key instanceof XAttributeContinuous) {
			if (value instanceof Double) {
				((XAttributeContinuous) clone).setValue((Double) value);
				return clone;
			}
			if (value instanceof String) {
				try {
					((XAttributeContinuous) clone).setValue(Double.valueOf((String) value));
					return clone;
				} catch (final NumberFormatException _) {
					// Ignore
				}
			}
		}
		if (key instanceof XAttributeDiscrete) {
			if (value instanceof Long) {
				((XAttributeDiscrete) clone).setValue((Long) value);
				return clone;
			}
			if (value instanceof Integer) {
				((XAttributeDiscrete) clone).setValue((Integer) value);
				return clone;
			}
			if (value instanceof String) {
				try {
					((XAttributeDiscrete) clone).setValue(Long.valueOf((String) value));
					return clone;
				} catch (final NumberFormatException _) {
					// Ignore
				}
			}
		}
		if (key instanceof XAttributeLiteral) {
			((XAttributeLiteral) clone).setValue("" + value);
			return clone;
		}
		return null;
	}

	private static Object get(final List<String> keys, final Map<String, Object> item) {
		final List<Object> result = new ArrayList<Object>(keys.size());
		if (keys != null) {
			for (final String key : keys) {
				final Object value = item.get(key);
				if (value != null) {
					result.add(value);
				}
			}
		}
		if (result.isEmpty())
			return null;
		if (result.size() == 1)
			return result.get(0);
		return result;
	}

	private static Date datify(final String string) {

		final Matcher matcher = KeyValueToLog.pattern.matcher(string);
		try {
			if (matcher.matches()) {
				final int d = Integer.parseInt(matcher.group(1));
				final int m = Integer.parseInt(matcher.group(2));
				int y = Integer.parseInt(matcher.group(3));
				if (y < 50) {
					y += 100;
				}
				if (y < 150) {
					y += 1900;
				}
				int h = 0, min = 0, s = 0;
				try {
					h = Integer.parseInt(matcher.group(5));
					min = Integer.parseInt(matcher.group(6));
					s = Integer.parseInt(matcher.group(8));
				} catch (final Exception _) {
					// Ignore
				}
				final GregorianCalendar gregorianCalendar = new GregorianCalendar(y, m - 1, d, h, min, s);
				return gregorianCalendar.getTime();
			} else {
				// assume xsDataTime
				return XAttributeTimestamp.FORMATTER.parseObject(string);
			}
		} catch (final Exception _) {
			// Mask
		}
		return null;
	}
	
}
