package org.processmining.logenhancement.enriching;

import java.util.List;
import java.util.Set;

import org.deckfour.xes.classification.XEventClass;
import org.deckfour.xes.classification.XEventClassifier;
import org.deckfour.xes.factory.XFactory;
import org.deckfour.xes.factory.XFactoryRegistry;
import org.deckfour.xes.info.XLogInfo;
import org.deckfour.xes.info.XLogInfoFactory;
import org.deckfour.xes.model.XAttribute;
import org.deckfour.xes.model.XAttributeBoolean;
import org.deckfour.xes.model.XAttributeContinuous;
import org.deckfour.xes.model.XAttributeDiscrete;
import org.deckfour.xes.model.XAttributeLiteral;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginCategory;
import org.processmining.framework.util.ui.widgets.helper.ProMUIHelper;
import org.processmining.framework.util.ui.widgets.helper.UserCancelledException;
import org.processmining.log.utils.XUtils;
import org.processmining.logenhancement.utils.LogEnhancementHelper;

import com.google.common.collect.ImmutableList;
import com.google.common.collect.Sets;

public class AddAttributePlugin {

	@Plugin(name = "Add Attribute to Event Class", parameterLabels = { "A log" }, //
	help = "Adds a new attribute to all events of a given event class.", categories = { PluginCategory.Enhancement }, returnLabels = {}, returnTypes = {}, userAccessible = true, mostSignificantResult = -1)
	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = " F. Mannhardt", email = "f.mannhardt@tue.nl")
	public void addAttribute(UIPluginContext context, XLog log) throws UserCancelledException {

		context.getProgress().setMaximum(log.size());

		XEventClassifier classifier = LogEnhancementHelper.queryEventClassifier(context, log);

		XLogInfo logInfo = XLogInfoFactory.createLogInfo(log, classifier);
		Set<XEventClass> eventNames = Sets.newTreeSet(logInfo.getEventClasses().getClasses());

		List<XEventClass> selectedEventClasses = ProMUIHelper.queryForObjects(context,
				"Please select the event classes that the new attribute should be added to", eventNames);

		String newAttributeName = ProMUIHelper.queryForString(context,
				"Please provide a name for the attribute to be created");

		Class<? extends XAttribute> newAttributeType = ProMUIHelper.queryForObject(context,
				"Please provide the type of the attribute to be created", ImmutableList.of(XAttributeLiteral.class,
						XAttributeDiscrete.class, XAttributeContinuous.class, XAttributeBoolean.class));

		String valueAsString = ProMUIHelper.queryForString(context, "Please provide the value of the new attribute");

		Boolean overwrite = ProMUIHelper.queryForObject(context, "Should existing attributes be overwritten?",
				new Boolean[] { Boolean.TRUE, Boolean.FALSE });

		XFactory factory = XFactoryRegistry.instance().currentDefault();

		doAddAttribute(factory, log, logInfo, selectedEventClasses, newAttributeName, newAttributeType, valueAsString,
				overwrite);

	}

	public void doAddAttribute(XFactory factory, XLog log, XLogInfo logInfo, List<XEventClass> selectedEventClasses,
			String newAttributeName, Class<? extends XAttribute> newAttributeType, String valueAsString,
			boolean overwrite) {
		for (XTrace t : log) {
			for (XEvent e : t) {
				if (selectedEventClasses.contains(logInfo.getEventClasses().getClassOf(e))) {
					if (overwrite || !e.getAttributes().containsKey(newAttributeName)) {
						if (newAttributeType == XAttributeLiteral.class) {
							XUtils.putAttribute(e,
									factory.createAttributeLiteral(newAttributeName, valueAsString, null));
						} else if (newAttributeType == XAttributeDiscrete.class) {
							XUtils.putAttribute(e, factory.createAttributeDiscrete(newAttributeName,
									Long.parseLong(valueAsString), null));
						} else if (newAttributeType == XAttributeContinuous.class) {
							XUtils.putAttribute(
									e,
									factory.createAttributeContinuous(newAttributeName,
											Double.parseDouble(valueAsString), null));
						} else if (newAttributeType == XAttributeBoolean.class) {
							XUtils.putAttribute(
									e,
									factory.createAttributeBoolean(newAttributeName,
											Boolean.parseBoolean(valueAsString), null));
						} else {
							throw new RuntimeException("Unkown type " + newAttributeType);
						}
					}
				}
			}
		}
	}

}
