package org.processmining.logenhancement.enriching;

import org.deckfour.xes.extension.std.XConceptExtension;
import org.deckfour.xes.factory.XFactory;
import org.deckfour.xes.factory.XFactoryRegistry;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.Progress;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginCategory;
import org.processmining.framework.util.ui.widgets.helper.ProMUIHelper;
import org.processmining.framework.util.ui.widgets.helper.UserCancelledException;
import org.processmining.log.utils.XUtils;

public class AddNextActivityPlugin {

	@Plugin(name = "Add Next Activity as Attribute to all Events", parameterLabels = { "A log" }, //
			help = "Adds a new attribute with the next activity to all events.", categories = {
					PluginCategory.Enhancement }, returnLabels = {}, returnTypes = {}, userAccessible = true, mostSignificantResult = -1)
	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = " Alfredo Bolt", email = "a.bolt@tue.nl")
	public void addNextActivity(UIPluginContext context, XLog log) throws UserCancelledException {

		Progress progBar = context.getProgress();
		progBar.setMinimum(0);
		progBar.setMaximum(log.size());
		progBar.setValue(0);

		String nextActivityKey = ProMUIHelper.queryForString(context, "Specify name of the attribute to be created");

		doAddNextActivity(progBar, log, nextActivityKey, XFactoryRegistry.instance().currentDefault());
	}

	public void doAddNextActivity(Progress progress, XLog log, String nextActivityKey, XFactory factory) {

		for (XTrace trace : log) {
			for (XEvent event : trace) {
				if (trace.indexOf(event) + 1 < trace.size()) {
					XEvent nextEvent = trace.get(trace.indexOf(event) + 1);
					XUtils.putAttribute(event, factory.createAttributeLiteral(nextActivityKey,
							XConceptExtension.instance().extractName(nextEvent), null));
				}
			}
			progress.inc();
		}
	}
}
