package org.processmining.logenhancement.filtering;

import java.util.HashSet;
import java.util.Random;
import java.util.Set;

import org.deckfour.xes.model.XLog;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.Progress;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginCategory;
import org.processmining.framework.util.ui.widgets.helper.ProMUIHelper;
import org.processmining.framework.util.ui.widgets.helper.UserCancelledException;
import org.processmining.plugins.utils.ProvidedObjectHelper;

public class RandomLogSamplePlugin {

	@Plugin(name = "Extract sample of traces (Random)", parameterLabels = { "A log" }, //			
			returnLabels = { "Random Sample Log" }, returnTypes = {
					XLog.class }, userAccessible = true, help = "Extracts a random sample of traces from the event log. The size of the sample can be given.", categories = {
							PluginCategory.Filtering }, mostSignificantResult = 1)
	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = " F. Mannhardt", email = "f.mannhardt@tue.nl")
	public XLog createRandomSampleByTraces(UIPluginContext context, XLog log) {
		final Progress progress = context.getProgress();

		try {
			int numberOfTraces = ProMUIHelper.queryForInteger(context,
					"Specify required sample size (Number of traces)");

			Integer seed = null;
			try {
				seed = ProMUIHelper.queryForInteger(context,
						"Specify seed value for the random sampling (leave empty for random seed)");
			} catch (NumberFormatException e) {
			}

			XLog sampleLog;
			if (seed == null) {
				sampleLog = createRandomSampleLog(log, numberOfTraces, progress);
			} else {
				sampleLog = createRandomSampleLog(log, numberOfTraces, progress, seed);
			}

			context.getFutureResult(0).setLabel(ProvidedObjectHelper.getProvidedObjectLabel(context, log)
					+ " - Random Sample of " + numberOfTraces);

			return sampleLog;
		} catch (NumberFormatException e) {
			ProMUIHelper.showErrorMessage(context, e.getMessage(), "Could not parse number");
			return null;
		} catch (UserCancelledException e) {
			context.log(e);
			return null;
		}
	}

	public static XLog createRandomSampleLog(XLog log, int numTraces, Progress progress) {
		return createRandomSampleLog(log, numTraces, progress, new Random());
	}

	public static XLog createRandomSampleLog(XLog log, int numTraces, Progress progress, int seed) {
		return createRandomSampleLog(log, numTraces, progress, new Random(seed));
	}

	public static XLog createRandomSampleLog(XLog log, int numTraces, Progress progress, Random random) {
		Set<Integer> traceIndexSet = new HashSet<Integer>(numTraces);
		// Prevent infinite loop, sample size has to be smaller than log size
		numTraces = Math.min(log.size(), numTraces);
		for (int i = 0; i < numTraces; i++) {
			boolean isAdded = false;
			while (!isAdded) {
				isAdded = traceIndexSet.add(random.nextInt(log.size()));
			}
		}
		return TraceSelectionPlugin.extractTracesByIndex(log, traceIndexSet, progress);
	}

}