package org.processmining.logenhancement.filtering;

import java.util.Iterator;
import java.util.ListIterator;
import java.util.Map.Entry;

import org.deckfour.xes.extension.std.XConceptExtension;
import org.deckfour.xes.extension.std.XOrganizationalExtension;
import org.deckfour.xes.extension.std.XTimeExtension;
import org.deckfour.xes.model.XAttribute;
import org.deckfour.xes.model.XAttributeMap;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.Progress;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginCategory;

/**
 * Removes all attributes with the value 'NOT_SET'. PLEASE NOTE: This filter
 * will update the original XLog instead of creating a new XLog.
 * 
 * @author F. Mannhardt
 * 
 */
public class RemoveNonStandardAttributesPlugin {

	private Progress progBar;

	@Plugin(name = "Remove all 'non-standard' attributes (In Place)", parameterLabels = { "Event Log" }, //
			categories = {
					PluginCategory.Filtering }, help = "Removes all attributes that are not known in one of the standard XES extensions such as concept, time, ...", returnLabels = {}, returnTypes = {}, userAccessible = true, mostSignificantResult = -1)
	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = " F. Mannhardt", email = "f.mannhardt@tue.nl")
	public void removeNonStandardAttributes(PluginContext context, XLog log) {

		progBar = context.getProgress();
		progBar.setMinimum(0);
		progBar.setMaximum(log.size());
		progBar.setValue(0);

		ListIterator<XTrace> traceIterator = log.listIterator();

		while (traceIterator.hasNext()) {
			XTrace trace = traceIterator.next();

			doRemoveNonStandardAttributes(trace.getAttributes());

			ListIterator<XEvent> eventIterator = trace.listIterator();
			while (eventIterator.hasNext()) {
				int eventIndex = eventIterator.nextIndex();
				XEvent event = eventIterator.next();

				doRemoveNonStandardAttributes(event.getAttributes());

				trace.set(eventIndex, event);
			}
			progBar.inc();
		}
	}

	private XAttributeMap doRemoveNonStandardAttributes(XAttributeMap attributes) {
		Iterator<Entry<String, XAttribute>> attrIter = attributes.entrySet().iterator();
		while (attrIter.hasNext()) {
			Entry<String, XAttribute> entry = attrIter.next();
			if (isNonStandard(entry.getKey())) {
				attrIter.remove();
			}
		}
		return attributes;
	}

	private boolean isNonStandard(String key) {
		return !(XConceptExtension.KEY_NAME.equals(key) || XConceptExtension.KEY_INSTANCE.equals(key)
				|| XTimeExtension.KEY_TIMESTAMP.equals(key) || XOrganizationalExtension.KEY_GROUP.equals(key)
				|| XOrganizationalExtension.KEY_RESOURCE.equals(key) || XOrganizationalExtension.KEY_ROLE.equals(key));
	}

}
