package org.processmining.logenhancement.filtering;

import java.util.Iterator;
import java.util.Set;

import org.deckfour.xes.factory.XFactory;
import org.deckfour.xes.factory.XFactoryRegistry;
import org.deckfour.xes.model.XAttributeMap;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.Progress;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginCategory;
import org.processmining.framework.util.ui.widgets.helper.ProMUIHelper;
import org.processmining.framework.util.ui.widgets.helper.UserCancelledException;
import org.processmining.log.utils.XUtils;

import com.google.common.collect.ImmutableSet;
import com.google.common.primitives.Ints;

public class TraceSelectionPlugin {

	@Plugin(name = "Extract traces by index in log", parameterLabels = { "A log" }, //
			returnLabels = { "Filtered Log by Index" }, returnTypes = {
					XLog.class }, help = "Creates a new log containing only the specified traces specified by their index in the log.", categories = {
							PluginCategory.Filtering }, userAccessible = true, mostSignificantResult = 1)
	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = " F. Mannhardt", email = "f.mannhardt@tue.nl")
	public XLog filterLog(UIPluginContext context, XLog log) {
		final Progress progress = context.getProgress();
		try {
			Set<Integer> traceIndexToKeep = ImmutableSet
					.<Integer>copyOf(Ints.asList(ProMUIHelper.queryForIntArray(context,
							"Specify Trace Indicies to be extracted (Separated by Comma: 1,2,3,...")));
			return extractTracesByIndex(log, traceIndexToKeep, progress);
		} catch (NumberFormatException e) {
			context.log(e);
			return null;
		} catch (UserCancelledException e) {
			context.log(e);
			return null;
		}
	}

	public static XLog extractTracesByIndex(XLog log, Set<Integer> traceIndexToKeep, Progress progress) {
		return extractTracesByIndex(log, traceIndexToKeep, XFactoryRegistry.instance().currentDefault(), progress);
	}

	public static XLog extractTracesByIndex(XLog log, Set<Integer> traceIndexToKeep, XFactory factory,
			Progress progress) {

		XLog newLog = XUtils.createLogFrom(log, factory);

		progress.setMaximum(log.size());

		int i = 0;
		for (Iterator<XTrace> iterator = log.iterator(); iterator.hasNext();) {
			XTrace t = iterator.next();
			if (traceIndexToKeep.contains(i)) {
				XTrace newTrace = factory.createTrace((XAttributeMap) t.getAttributes().clone());
				for (XEvent e : t) {
					newTrace.add(factory.createEvent((XAttributeMap) e.getAttributes().clone()));
				}
				newLog.add(newTrace);
			}
			i++;
			progress.inc();
		}

		return newLog;
	}

}
