package org.processmining.logenhancement.filtering.attributes;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import javax.swing.BoxLayout;
import javax.swing.DefaultComboBoxModel;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.JPanel;

import org.deckfour.uitopia.api.event.TaskListener.InteractionResult;
import org.deckfour.xes.classification.XEventClass;
import org.deckfour.xes.factory.XFactory;
import org.deckfour.xes.factory.XFactoryRegistry;
import org.deckfour.xes.model.XAttributeLiteral;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.framework.util.ui.widgets.BorderPanel;
import org.processmining.framework.util.ui.widgets.ProMComboBox;
import org.processmining.framework.util.ui.widgets.ProMList;
import org.processmining.framework.util.ui.widgets.ProMPropertiesPanel;
import org.processmining.framework.util.ui.widgets.ProMTextField;
import org.processmining.framework.util.ui.widgets.WidgetColors;

import au.com.bytecode.opencsv.CSVReader;

import com.fluxicon.slickerbox.components.SlickerTabbedPane;
import com.fluxicon.slickerbox.factory.SlickerFactory;

/**
 * Plug-in that filters/manipulates attributes of events in the Log/Trace
 * 
 * @author fmannhardt
 * 
 */
public class AttributeEditorPanel extends BorderPanel {

	private static final String PANEL_TITLE = "Filter Attributes of Events in the Log";

	private final class AttributeListener implements ActionListener {

		public void actionPerformed(ActionEvent e) {
			fillValueCB((String) attributeCB.getSelectedItem());
		}

	}

	private final class FilterOnListener implements ActionListener {

		public void actionPerformed(ActionEvent e) {
			if (filterOnCB.getSelectedItem() == AttributeFilterByEnum.ATTRIBUTE_NAME) {
				valueCB.setVisible(false);
			} else {
				valueCB.setVisible(true);
				fillValueCB((String) attributeCB.getSelectedItem());
			}
		}

	}

	public class AddFilterListener implements ActionListener {

		public void actionPerformed(ActionEvent e) {
			AttributeEditorSelection currentFilterSelection = new AttributeEditorSelection();
			currentFilterSelection.setAttributeName(getFilterAttributeName());
			currentFilterSelection.setAttributeValue(getFilterAttributeValue());
			currentFilterSelection.setEventClass(getFilterEventNameValue());
			currentFilterSelection.setFilterOption(getFilterOnOption());
			currentFilterSelection.setReplacementValue(getReplacementValue());
			filterListModel.addElement(currentFilterSelection);
		}

	}

	public class ImportMappingListener implements ActionListener {

		public void actionPerformed(ActionEvent event) {
			JFileChooser fileDialog = new JFileChooser();
			int returnVal = fileDialog.showOpenDialog(AttributeEditorPanel.this);
			if (returnVal == JFileChooser.APPROVE_OPTION) {
				try (CSVReader reader = new CSVReader(new FileReader(fileDialog.getSelectedFile()))) {
					
					XFactory f = XFactoryRegistry.instance().currentDefault();

					String[] next;
					while ((next = reader.readNext()) != null) {

						AttributeEditorSelection importedFilter = new AttributeEditorSelection();
						if (next.length < 4) {
							JOptionPane
									.showMessageDialog(
											AttributeEditorPanel.this,
											"Mapping files is missing columns. Format should be [eventclass],[attribute],[value],[replacement],[additionalAttribute1_name],[additionalAttribute1_value],...",
											"Wrong format", JOptionPane.ERROR_MESSAGE);
							return;
						}

						String eventClass = next[0];
						String attributeName = next[1];
						String attributeValue = next[2];
						String replacement = next[3];

						if (!eventClass.isEmpty()) {
							for (XEventClass ec : logAdapter.getAllEventClasses()) {
								if (ec.getId().equals(eventClass)) {
									importedFilter.setEventClass(ec);
								}
							}
							if (importedFilter.getEventClass() == null) {
								JOptionPane.showMessageDialog(AttributeEditorPanel.this, "Unkown event class "
										+ eventClass, "Wrong format", JOptionPane.ERROR_MESSAGE);
							}
						}
						importedFilter.setAttributeName(attributeName);
						importedFilter.setAttributeValue(attributeValue);
						importedFilter.setFilterOption(AttributeFilterByEnum.ATTRIBUTE_NAME_VALUE);
						importedFilter.setReplacementValue(replacement);
						
						if (next.length > 4) {
							
							for (int i = 4; i < next.length; i = i + 2) {
								String additionalName = next[i];
								String additionalValue = next[i+1];
								XAttributeLiteral additionalAttribute = f.createAttributeLiteral(additionalName, additionalValue, null);
								importedFilter.getAdditionalAttributes().add(additionalAttribute);
							}
							
						}

						filterListModel.addElement(importedFilter);
					}

				} catch (IOException e) {
					JOptionPane.showMessageDialog(AttributeEditorPanel.this, e.getMessage(), "Failed to load file",
							JOptionPane.ERROR_MESSAGE);
				}
			}
		}

	}

	private static final long serialVersionUID = -9142356217618321756L;

	private LogAdapter logAdapter;

	private ProMComboBox<AttributeFilterByEnum> filterOnCB;

	private ProMComboBox<String> attributeCB;

	private ProMComboBox<Object> valueCB;

	private ProMComboBox<XEventClass> eventClassCB;

	private DefaultListModel<AttributeEditorSelection> filterListModel;

	private ProMTextField replacementValue;

	public AttributeEditorPanel(LogAdapter logAdapter) {
		super(0, 0);
		this.logAdapter = logAdapter;
		init();
	}

	/**
	 * Show the dialog
	 * 
	 * @param context
	 * @return
	 */
	public InteractionResult getUserChoice(UIPluginContext context) {
		return context.showConfiguration(PANEL_TITLE, this);
	}

	/**
	 * @return the user selected filters
	 */
	public List<AttributeEditorSelection> getFilterSelection() {
		List<AttributeEditorSelection> selectionList = new ArrayList<AttributeEditorSelection>();
		for (int i = filterListModel.size() - 1; i >= 0; i--) {
			selectionList.add(filterListModel.elementAt(i));
		}
		return selectionList;
	}

	private void init() {
		SlickerTabbedPane mainTabbedPane = SlickerFactory.instance().createTabbedPane("", WidgetColors.COLOR_LIST_BG,
				WidgetColors.COLOR_LIST_FG, Color.GREEN);
		setLayout(new BorderLayout());
		mainTabbedPane.setPreferredSize(new Dimension(200, 600));
		add(mainTabbedPane);

		JPanel panel = SlickerFactory.instance().createRoundedPanel();
		panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));
		mainTabbedPane.addTab(PANEL_TITLE, panel);

		ProMPropertiesPanel filterAllEventsPanel = new ProMPropertiesPanel(PANEL_TITLE);
		filterOnCB = new ProMComboBox<AttributeFilterByEnum>(AttributeFilterByEnum.values());
		filterOnCB.setSelectedItem(AttributeFilterByEnum.ATTRIBUTE_NAME);
		filterOnCB.addActionListener(new FilterOnListener());
		attributeCB = new ProMComboBox<String>(logAdapter.getAllEventAttributes());
		attributeCB.addActionListener(new AttributeListener());
		valueCB = new ProMComboBox<Object>(new String[] { "none" });
		Set<XEventClass> allEventClasses = logAdapter.getAllEventClasses();
		allEventClasses.add(new XEventClass("", Integer.MIN_VALUE));
		eventClassCB = new ProMComboBox<XEventClass>(allEventClasses);

		filterAllEventsPanel.addProperty("Choose Event (empty = all)", eventClassCB);
		filterAllEventsPanel.addProperty("Filter by", filterOnCB);
		filterAllEventsPanel.addProperty("Attribute Name", attributeCB);
		filterAllEventsPanel.addProperty("Attribute Value", valueCB);

		replacementValue = new ProMTextField();
		filterAllEventsPanel.addProperty("Replacement Value", replacementValue);

		panel.add(filterAllEventsPanel);

		JPanel buttonPanel = new JPanel();
		buttonPanel.setLayout(new BoxLayout(buttonPanel, BoxLayout.X_AXIS));
		buttonPanel.setBackground(null);

		JButton addFilterBtn = SlickerFactory.instance().createButton("Add Filter");
		addFilterBtn.addActionListener(new AddFilterListener());
		buttonPanel.add(addFilterBtn);

		JButton importMappingBtn = SlickerFactory.instance().createButton("Import Mapping");
		importMappingBtn.addActionListener(new ImportMappingListener());
		buttonPanel.add(importMappingBtn);

		panel.add(buttonPanel);

		filterListModel = new DefaultListModel<AttributeEditorSelection>();
		ProMList<AttributeEditorSelection> filterList = new ProMList<AttributeEditorSelection>("Added Filters",
				filterListModel);
		panel.add(filterList);
	}

	private void fillValueCB(String selectedAttribute) {
		Set<Object> availableValues = logAdapter.getAllEventValues(selectedAttribute);
		valueCB.setModel(new DefaultComboBoxModel<Object>(availableValues.toArray(new Object[] {})));
	}

	private AttributeFilterByEnum getFilterOnOption() {
		return (AttributeFilterByEnum) filterOnCB.getSelectedItem();
	}

	private String getFilterAttributeName() {
		return (String) attributeCB.getSelectedItem();
	}

	private Object getFilterAttributeValue() {
		if (getFilterOnOption() == AttributeFilterByEnum.ATTRIBUTE_NAME_VALUE) {
			return valueCB.getSelectedItem();
		}
		return null;

	}

	private XEventClass getFilterEventNameValue() {
		XEventClass eventName = (XEventClass) eventClassCB.getSelectedItem();
		if (eventName.getIndex() == Integer.MIN_VALUE) {
			return null;
		} else {
			return eventName;
		}
	}

	public String getReplacementValue() {
		return replacementValue.getText();
	}

}
