package org.processmining.logenhancement.merge.alignmentbased;

import java.util.Date;
import java.util.Map;
import java.util.Map.Entry;

import org.deckfour.xes.extension.std.XTimeExtension;
import org.deckfour.xes.model.XAttribute;
import org.deckfour.xes.model.XAttributeMap;
import org.deckfour.xes.model.XEvent;
import org.processmining.plugins.DataConformance.Utility;
import org.processmining.plugins.balancedconformance.dataflow.datastate.DataState;
import org.processmining.plugins.balancedconformance.dataflow.datastate.DataStateFactory;
import org.processmining.plugins.balancedconformance.dataflow.datastate.DataStateHelper;
import org.processmining.plugins.balancedconformance.mapping.StringDiscretizer;

import com.google.common.collect.ComparisonChain;

final class EventEntryImpl implements EventEntry {

	private final XEvent event;
	private final Date time;
	private final int eventIdx;
	private DataState attributes;

	EventEntryImpl(XEvent event, int eventIdx, DataStateFactory stateFactoy,
			@SuppressWarnings("rawtypes") Map<String, Class> varType, Map<String, String> variableMapping,
			StringDiscretizer discretizer) {
		this(event, eventIdx);
		this.attributes = convertAttributesToDataState(event, stateFactoy, varType, variableMapping, discretizer);
	}

	@SuppressWarnings("rawtypes")
	private DataState convertAttributesToDataState(XEvent event, DataStateFactory stateFactoy,
			Map<String, Class> varType, Map<String, String> variableMapping, StringDiscretizer discretizer) {
		DataState state = stateFactoy.newDataState();
		XAttributeMap attributeMap = event.getAttributes();
		for (Entry<String, Class> entry : varType.entrySet()) {
			String varName = entry.getKey();
			XAttribute attribute = attributeMap.get(variableMapping.get(varName));
			if (attribute != null) {
				Class<?> expectedType = entry.getValue();
				Object value = org.processmining.plugins.DataConformance.Utility.getValue(attribute);
				if (!Utility.isAssignableToExpectedType(value, expectedType)) {
					throw new RuntimeException("Type mismatch between Log and DPN-net: " + expectedType
							+ " excpected for " + value);
				}
				// Store both as prime and non prime					
				DataStateHelper.storeValue(state, varName, value, expectedType, discretizer);
				DataStateHelper.storeValue(state, varName.concat("'"), value, expectedType, discretizer);
			}
		}
		state.makeStateImmutable();
		return state;
	}

	public EventEntryImpl(XEvent event, int eventIdx) {
		this.event = event;
		this.eventIdx = eventIdx;
		this.time = XTimeExtension.instance().extractTimestamp(event);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.processmining.plugins.fm.project4.EEntry#compareTo(org.processmining
	 * .plugins.fm.project4.AlignmentBasedLogMerger.EventEntry)
	 */
	@Override
	public int compareTo(EventEntry o) {
		return ComparisonChain.start().compare(time, o.getDate()).compare(eventIdx, o.getEventIndex()).result();
	}

	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + eventIdx;
		return result;
	}

	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		EventEntryImpl other = (EventEntryImpl) obj;
		if (eventIdx != other.eventIdx)
			return false;
		return true;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.processmining.plugins.fm.project4.EEntry#getEvent()
	 */
	@Override
	public XEvent getEvent() {
		return event;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.processmining.plugins.fm.project4.EEntry#getDate()
	 */
	@Override
	public Date getDate() {
		return time;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.processmining.plugins.fm.project4.EEntry#getEventIndex()
	 */
	@Override
	public int getEventIndex() {
		return eventIdx;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.processmining.plugins.fm.project4.EEntry#getAttributes()
	 */
	@Override
	public DataState getAttributes() {
		return attributes;
	}

}