package org.processmining.logenhancement.merge.alignmentbased;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import org.deckfour.xes.extension.std.XConceptExtension;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.plugins.DataConformance.Alignment;

class TraceSetImpl implements TraceSet {

	private Map<String, XTrace> traceByName;
	private Set<TraceEntry> retrySet;

	public TraceSetImpl(XLog masterLog) {
		super();
		traceByName = new HashMap<String, XTrace>(masterLog.size());
		retrySet = new HashSet<TraceEntry>();
		putAll(masterLog);
	}

	private void putAll(XLog log) {
		XConceptExtension conceptExtension = XConceptExtension.instance();
		for (XTrace t : log) {
			String traceName = conceptExtension.extractName(t);
			assert traceName != null : "Trace should have a Name at least!";
			traceByName.put(traceName, t);
		}
	}

	public XTrace getByName(String name) {
		return traceByName.get(name);
	}

	public boolean markRetry(XTrace t, Alignment alignment, Candidates candidates) {
		return retrySet.add(new TraceEntry(t, alignment, candidates));
	}

	public boolean hasRetries() {
		return !retrySet.isEmpty();
	}

	public int retryCount() {
		return retrySet.size();
	}

	public Iterator<TraceEntry> retryIterator() {
		return retrySet.iterator();
	}

}