package org.processmining.logenhancement.transforming;

import java.util.Iterator;
import java.util.ListIterator;
import java.util.Map.Entry;

import org.deckfour.xes.extension.std.XConceptExtension;
import org.deckfour.xes.extension.std.XLifecycleExtension;
import org.deckfour.xes.extension.std.XTimeExtension;
import org.deckfour.xes.model.XAttribute;
import org.deckfour.xes.model.XAttributeMap;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.Progress;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginCategory;

/**
 * Plug-in that copies all trace level attributes (except for concept:name,
 * lifecycle:transition and time:timestamp) to the 1st event of the trace.
 * PLEASE NOTE: This filter will update the original XLog instead of creating a
 * new XLog.
 * 
 * @author F. Mannhardt
 * 
 */
public class CopyTraceAttributesToFirstEventPlugin {

	private Progress progBar;

	@Plugin(name = "Copy trace attributes to the 1st event (In place)", parameterLabels = {
			"Event Log" }, help = "Plug-in that copies all trace level attributes (except for concept:name, lifecycle:transition and time:timestamp) to the 1st event of the trace. "
					+ "PLEASE NOTE: This filter will update the original XLog instead of creating a new XLog.", categories = {
							PluginCategory.Enhancement }, returnLabels = {}, returnTypes = {}, userAccessible = true, mostSignificantResult = -1)
	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = " F. Mannhardt", email = "f.mannhardt@tue.nl")
	public void copyAttributesToFirstEvent(PluginContext context, XLog log) {

		progBar = context.getProgress();
		progBar.setMinimum(0);
		progBar.setMaximum(log.size()); // two pass 
		progBar.setValue(0);

		doCopyAttributeToFirstEvent(log);

	}

	public void doCopyAttributeToFirstEvent(XLog log) {
		ListIterator<XTrace> traceIterator = log.listIterator();

		while (traceIterator.hasNext()) {

			XTrace trace = traceIterator.next();

			XAttributeMap traceAttr = trace.getAttributes();
			XEvent event = trace.get(0);

			Iterator<Entry<String, XAttribute>> attributeIterator = traceAttr.entrySet().iterator();
			while (attributeIterator.hasNext()) {
				Entry<String, XAttribute> entry = attributeIterator.next();
				if (isSpecialAttribute(entry))
					event.getAttributes().put(entry.getKey(), entry.getValue());
			}
			trace.set(0, event);
			onProgress();
		}
	}

	private boolean isSpecialAttribute(Entry<String, XAttribute> entry) {
		return !entry.getKey().equals(XConceptExtension.KEY_NAME)
				&& !entry.getKey().equals(XTimeExtension.KEY_TIMESTAMP)
				&& !entry.getKey().equals(XLifecycleExtension.KEY_TRANSITION);
	}

	public void onProgress() {
		progBar.inc();
	}

}
