package org.processmining.logenhancement.transforming;

import java.util.Collections;
import java.util.Comparator;
import java.util.Date;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Objects;
import java.util.Set;

import org.deckfour.xes.extension.std.XTimeExtension;
import org.deckfour.xes.model.XAttribute;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.Progress;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginCategory;
import org.processmining.framework.util.ui.widgets.helper.ProMUIHelper;
import org.processmining.framework.util.ui.widgets.helper.UserCancelledException;

import com.google.common.collect.Maps;
import com.google.common.collect.Ordering;

public class MergeTracesWithSameAttribute {

	@Plugin(name = "Merge traces based on attribute (In place)", parameterLabels = {}, //	
	returnLabels = {}, returnTypes = {}, userAccessible = true, mostSignificantResult = -1, //
	categories = { PluginCategory.Enhancement }, help = "Merges those traces of an evvent log, which contain a trace attribute with the same value.")
	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = " F. Mannhardt", email = "f.mannhardt@tue.nl")
	public void mergeTracesWithSameAttribute(UIPluginContext context, XLog log) throws UserCancelledException {

		final Progress progress = context.getProgress();
		progress.setMaximum(log.size());

		Set<String> attributes = new HashSet<>();
		for (XTrace trace : log) {
			attributes.addAll(trace.getAttributes().keySet());
		}

		String attributeKey = ProMUIHelper.queryForObject(context, "Select trace attribute", attributes);

		Map<String, XTrace> traceMap = Maps.newHashMapWithExpectedSize(log.size());
		
		for (Iterator<XTrace> iterator = log.iterator(); iterator.hasNext();) {
			XTrace trace = iterator.next();
			
			XAttribute attribute = trace.getAttributes().get(attributeKey);
			
			XTrace existingTrace = traceMap.get(attribute.toString());
			if (existingTrace != null) {
				iterator.remove();
				existingTrace.addAll(trace);
				Collections.sort(existingTrace, new Comparator<XEvent>() {

					public int compare(XEvent o1, XEvent o2) {
						Date t1 = XTimeExtension.instance().extractTimestamp(o1);
						Date t2 = XTimeExtension.instance().extractTimestamp(o2);
						return Objects.compare(t1, t2, Ordering.<Date>natural());
					}
				});
				context.log("Merging based on "+attribute.toString());
			} else {
				traceMap.put(attribute.toString(), trace);
			}

			progress.inc();
			
			if (progress.isCancelled()) {
				return;
			}
		}

	}

}
