package org.processmining.logenhancement.transforming;

import java.text.DateFormat;
import java.util.Map;

import org.deckfour.xes.model.XAttribute;
import org.deckfour.xes.model.XLog;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.log.repair.RepairAttributeDataType;
import org.processmining.log.repair.RepairAttributeDataType.ReviewCallback;

/**
 * Tries to automatically guess the data type of all XES attributes and updates
 * the log accordingly.
 * <p>
 * PLEASE NOTE: This filter will update the original XLog instead of creating a
 * new XLog, to be able to process huge logs without exhausting the available
 * memory.
 * 
 * @author F. Mannhardt
 * 
 */
public final class RepairAttributeTypesPlugin {

	public RepairAttributeTypesPlugin() {
		super();
	}

	@Plugin(name = "Repair Type of Event Attributes in the Log (Discrete, Continuous, Boolean, Date)", parameterLabels = { "Event Log" }, returnLabels = {}, returnTypes = {}, userAccessible = true, mostSignificantResult = -1, 
			help = "Repairs the data type of event attributes in the log. For each attribute the correct XES datatype is guessed and applied.")
	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = " F. Mannhardt", email = "f.mannhardt@tue.nl")
	public void repairEventAttributeDataTypes(final UIPluginContext context, XLog log) {
		
		RepairAttributeDataType repairAttributeDataType = new RepairAttributeDataType();

		Iterable<? extends DateFormat> dateFormats = RepairAttributeDataType.queryDateFormats(context);

		repairAttributeDataType.doRepairEventAttributes(context, log, dateFormats, new ReviewCallback() {
			
			public Map<String, Class<? extends XAttribute>> reviewDataTypes(
					Map<String, Class<? extends XAttribute>> guessedDataTypes) {
				return RepairAttributeDataType.queryCustomDataTypes(context, guessedDataTypes);
			}
			
		});
		
	}
	
	@Plugin(name = "Repair Type of Trace Attributes in the Log (Discrete, Continuous, Boolean, Date)", parameterLabels = { "Event Log" },
			returnLabels = {}, returnTypes = {}, userAccessible = true, mostSignificantResult = -1, 
			help = "Repairs the data type of trace attributes in the log. For each attribute the correct XES datatype is guessed and applied.")
	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = " F. Mannhardt", email = "f.mannhardt@tue.nl")
	public void repairTraceAttributeDataTypes(final UIPluginContext context, XLog log) {

		RepairAttributeDataType repairAttributeDataType = new RepairAttributeDataType();

		Iterable<? extends DateFormat> dateFormats = RepairAttributeDataType.queryDateFormats(context);

		repairAttributeDataType.doRepairTraceAttributes(context, log, dateFormats, new ReviewCallback() {
			
			public Map<String, Class<? extends XAttribute>> reviewDataTypes(
					Map<String, Class<? extends XAttribute>> guessedDataTypes) {
				return RepairAttributeDataType.queryCustomDataTypes(context, guessedDataTypes);
			}
			
		});
		
	}

}
