package org.processmining.logenhancement.view;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.util.Collection;
import java.util.Iterator;
import java.util.Set;

import javax.swing.SwingWorker;

import org.deckfour.xes.model.XTrace;

import com.google.common.collect.ImmutableSet;

final class ActionRemoveFromDetail implements ActionListener, KeyListener {

	private final LogViewDetail detailView;
	private final LogViewMaster masterView;
	private final Collection<XTrace> traceCollection;

	ActionRemoveFromDetail(LogViewMaster masterView, LogViewDetail detailView, Collection<XTrace> traceCollection) {
		this.masterView = masterView;
		this.detailView = detailView;
		this.traceCollection = traceCollection;
	}

	public void actionPerformed(ActionEvent e) {
		removeTraces();
	}

	private void removeTraces() {

		final Set<XTrace> selection = ImmutableSet.copyOf(detailView.getDetailList().getList().getSelectedValuesList());

		masterView.getMasterList().beforeUpdate();
		detailView.getDetailList().clear();

		new SwingWorker<Void, Void>() {

			protected Void doInBackground() throws Exception {
				for (Iterator<XTrace> iterator = traceCollection.iterator(); iterator.hasNext();) {
					XTrace trace = iterator.next();
					if (selection.contains(trace)) {
						iterator.remove();
					}
				}
				return null;
			}

			protected void done() {
				masterView.getMasterList().afterUpdate();
				detailView.getDetailList().clear();
				masterView.reloadTraces(traceCollection, masterView.getCurrentGroupingFunction());				
			}

		}.execute();

	}

	public void keyTyped(KeyEvent e) {
	}

	public void keyPressed(KeyEvent e) {
		if (e.getKeyCode() == KeyEvent.VK_DELETE) {
			removeTraces();
		}
	}

	public void keyReleased(KeyEvent e) {
	}
}