package org.processmining.logenhancement.view;

import java.awt.Color;
import java.util.Map;

import org.deckfour.xes.classification.XEventClasses;
import org.deckfour.xes.model.XAttribute;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XTrace;
import org.processmining.framework.util.ui.widgets.ColorScheme;
import org.processmining.log.utils.XUtils;

import com.google.common.collect.HashMultiset;
import com.google.common.collect.Maps;
import com.google.common.collect.Multiset;
import com.google.common.collect.Multisets;

final class ColoringModeByAttribute implements ColoringMode {

	private final String attribute;

	public ColoringModeByAttribute(String attribute) {
		this.attribute = attribute;
	}

	public String getName() {
		return "Color by " + attribute;
	}

	public EventColoring createColoring(Iterable<XTrace> traces, XEventClasses eventClasses) {
		final Map<Object, Color> colorMap = Maps.newHashMap();
		Multiset<Object> values = HashMultiset.create();
		for (XTrace trace : traces) {
			for (XEvent e : trace) {
				XAttribute attr = e.getAttributes().get(attribute);
				if (attr != null) {
					values.add(XUtils.getAttributeValue(attr));
				}
			}
		}
		int i = 0;
		for (Object obj : Multisets.copyHighestCountFirst(values).elementSet()) {
			colorMap.put(obj, ColorScheme.COLOR_BREWER_12CLASS_PAIRED.getColor(i++));
		}
		return new EventColoring() {

			public Color getColor(XEvent event) {
				return colorMap.get(XUtils.getAttributeValue(event.getAttributes().get(attribute)));
			}
		};
	}

	public String toString() {
		return getName();
	}

}