package org.processmining.logenhancement.view;

import java.util.Collections;
import java.util.Comparator;

import org.processmining.framework.util.ui.widgets.traceview.ProMTraceView.Event;

import com.google.common.collect.ComparisonChain;
import com.google.common.collect.Multiset;
import com.google.common.collect.Multiset.Entry;

enum GroupOrdering implements Comparator<Multiset.Entry<XESTraceVariant>> {
	COUNT_ASC("Sort by Count (Ascending)") {
		@Override
		public int compare(final Entry<XESTraceVariant> a, final Entry<XESTraceVariant> b) {
			return ComparisonChain.start().compare(a.getCount(), b.getCount()).result();
		}
	},
	COUNT_DESC("Sort by Count (Descending)") {
		@Override
		public int compare(final Entry<XESTraceVariant> a, final Entry<XESTraceVariant> b) {
			return Collections.reverseOrder(COUNT_ASC).compare(a, b);
		}
	},
	LENGTH_ASC("Sort by Length (Ascending)") {
		@Override
		public int compare(final Entry<XESTraceVariant> a, final Entry<XESTraceVariant> b) {
			return ComparisonChain.start().compare(a.getElement().size(), b.getElement().size()).result();
		}
	},
	LENGTH_DESC("Sort by Length (Descending)") {
		@Override
		public int compare(final Entry<XESTraceVariant> a, final Entry<XESTraceVariant> b) {
			return Collections.reverseOrder(LENGTH_ASC).compare(a, b);
		}
	},
	PREFIX("Sort by Prefix") {
		@Override
		public int compare(Entry<XESTraceVariant> o1, Entry<XESTraceVariant> o2) {

			if (o1 == o2) {
				return 0;
			}

			XESTraceVariant element1 = o1.getElement();
			XESTraceVariant element2 = o2.getElement();
			int len1 = element1.size();
			int len2 = element2.size();
			int min = Math.min(len1, len2);

			int i = 0;
			while (i < min) {
				Event event1 = element1.get(i);
				Event event2 = element2.get(i);
				int labelDiff = event1.getLabel().compareTo(event2.getLabel());
				if (labelDiff != 0) {
					return labelDiff;
				}
				i++;
			}

			return len1 - len2;
		}
	},
	COUNT_PREFIX("Sort by Count & Prefix") {

		public int compare(Entry<XESTraceVariant> o1, Entry<XESTraceVariant> o2) {
			return ComparisonChain.start().compare(o1, o2, COUNT_DESC).compare(o1, o2, PREFIX).result();
		}

	};

	private String description;

	private GroupOrdering(String description) {
		this.description = description;
	}

	@Override
	public String toString() {
		return description;
	}
}